--
-- Volto1320t Combine

-- @date  11/07/09
--

Volto1320t = {};

function Volto1320t.prerequisitesPresent(specializations)
    return true;
end;

function Volto1320t:load(xmlFile)

    self.groundReferenceThreshold = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.groundReferenceNode#threshold"), 0.2);
    self.groundReferenceNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.groundReferenceNode#index"));
    if self.groundReferenceNode == nil then
        self.groundReferenceNode = self.components[1].node;
    end;

    self.rotors = {};

    local psFile = getXMLString(xmlFile, "vehicle.rotors.rotor(1)#index");
    if psFile ~= nil then
        local i=0;
        while true do
            local baseName = string.format("vehicle.rotors.rotor(%d)", i);
            local node = {};
            node.index = getXMLString(xmlFile, baseName.. "#index");
            node.direction = getXMLInt(xmlFile, baseName.. "#direction");
            if node.index == nil then
                break;
            end;
            node.index = Utils.indexToObject(self.components, node.index);
            if node ~= nil then
                table.insert(self.rotors, node);
            end;
            i = i+1;
        end;
    end;

    local numTedderDropAreas = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.tedderDropAreas#count"), 0);
    if numTedderDropAreas ~= table.getn(self.cuttingAreas) then
        print("Warning: number of cutting areas and drop areas should be equal");
    end;
    self.tedderDropAreas = {}
    for i=1, numTedderDropAreas do
        self.tedderDropAreas[i] = {};
        local areanamei = string.format("vehicle.tedderDropAreas.tedderDropArea%d", i);
        self.tedderDropAreas[i].start = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#startIndex"));
        self.tedderDropAreas[i].width = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#widthIndex"));
        self.tedderDropAreas[i].height = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#heightIndex"));
    end;

    local numCuttingAreas = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.cuttingAreas#count"), 0);
    for i=1, numCuttingAreas do
        local areanamei = string.format("vehicle.cuttingAreas.cuttingArea%d", i);
        self.cuttingAreas[i].foldMinLimit = Utils.getNoNil(getXMLFloat(xmlFile, areanamei .. "#foldMinLimit"), 0);
        self.cuttingAreas[i].foldMaxLimit = Utils.getNoNil(getXMLFloat(xmlFile, areanamei .. "#foldMaxLimit"), 1);
        self.cuttingAreas[i].grassParticleSystemIndex = getXMLInt(xmlFile, areanamei .. "#particleSystemIndex");
    end;

    self.grassParticleSystems = {};
    local i=0;
    while true do
        local baseName = string.format("vehicle.grassParticleSystems.grassParticleSystem(%d)", i);
		local nodei = Utils.indexToObject(self.components, "0>");
        local particleSystem = {};
        particleSystem.ps = {};
        local ps = Utils.loadParticleSystem(xmlFile, particleSystem.ps, baseName, nodei, false, nil, self.baseDirectory)
        if ps == nil then
            break;
        end;
        particleSystem.disableTime = 0;
        table.insert(self.grassParticleSystems, particleSystem);
        i = i+1;
    end;

    local tedderSound = getXMLString(xmlFile, "vehicle.tedderSound#file");
    if tedderSound ~= nil and tedderSound ~= "" then
        tedderSound = Utils.getFilename(tedderSound, self.baseDirectory);
        self.tedderSound = createSample("tedderSound");
        self.tedderSoundEnabled = false;
        loadSample(self.tedderSound, tedderSound, false);
        self.tedderSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.tedderSound#pitchOffset"), 1);
        self.tedderSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.tedderSound#volume"), 1.0);
    end;
	local Quot = 16/9;

	local counterpath = Utils.getFilename("Panel.png", self.baseDirectory);
	local counterpath2 = Utils.getFilename("counterWarning.png", self.baseDirectory);
	local hudPDAPoxX = 0.0;
    local hudPDAPoxY = 0.0; 
    local hudPDAHeight = 0.420;
    local hudPDAWidth = hudPDAHeight/Quot
	self.hudBasePoxX = {};
	self.hudBasePoxY = {};
	self.hudBasePoxX[0] = hudPDAPoxX+0.0835;
	self.hudBasePoxY[0] = hudPDAPoxY+0.1800;		
	self.hudBasePoxX[1] = hudPDAPoxX+0.0000;
	self.hudBasePoxY[1] = hudPDAPoxY+0.0845;
	self.hudBasePoxX[1] = hudPDAPoxX+0.0360;
	self.hudBasePoxY[1] = hudPDAPoxY+0.2320;
	self.hudBasePoxX[2] = hudPDAPoxX+0.0360;
	self.hudBasePoxY[2] = hudPDAPoxY+0.2035;
	self.hudBasePoxX[3] = hudPDAPoxX+0.0365;
	self.hudBasePoxY[3] = hudPDAPoxY+0.1730;
	self.hudBasePoxX[4] = hudPDAPoxX+0.0380;
	self.hudBasePoxY[4] = hudPDAPoxY+0.1440;
	self.hudBasePoxX[5] = hudPDAPoxX+0.0395;
	self.hudBasePoxY[5] = hudPDAPoxY+0.1155;
	self.hudBasePoxX[6] = hudPDAPoxX+0.1890;
	self.hudBasePoxY[6] = hudPDAPoxY+0.2315;
	self.hudBasePoxX[7] = hudPDAPoxX+0.1890;
	self.hudBasePoxY[7] = hudPDAPoxY+0.2025;
	self.hudBasePoxX[8] = hudPDAPoxX+0.1890;
	self.hudBasePoxY[8] = hudPDAPoxY+0.1730;
	self.hudBasePoxX[9] = hudPDAPoxX+0.1870;
	self.hudBasePoxY[9] = hudPDAPoxY+0.1440;
	self.hudBasePoxX[10] = hudPDAPoxX+0.1860;
	self.hudBasePoxY[10] = hudPDAPoxY+0.1155;
    self.hudPDAOverlay = Overlay:new("hudPDAControl", counterpath, hudPDAPoxX, hudPDAPoxY, hudPDAWidth, hudPDAHeight);	
	self.hudPDAOverlayL = Overlay:new("hudPDAControl2", counterpath2, hudPDAPoxX, hudPDAPoxY, hudPDAWidth, hudPDAHeight);
	self.hudActive = true;
	
    self.isTurnedOn = false;
    self.wasToFast = false;
	
	self.fixAttacher = true;	
	self.Go.left = false;
	self.Go.righ = true;
	self.Go.back = false;
	self.jointLimiter = false;
end;



function Volto1320t:update(dt)		
	self.wasToFast = false;
	if self:getIsActive() then
		if self:getIsActiveForInput() then			
			if InputBinding.hasEvent(InputBinding.Volto1320tOverlay) then
				self.hudActive = not self.hudActive;
			end;
		end;
	end;
	if InputBinding.hasEvent(InputBinding.Volto_trsp) and self:getIsActiveForInput() and not self.Go.back then	
		self.Speed.right = 0.30;
		self.Go.right = not self.Go.right;
		self.Done.right = true; 	
		self.Speed.left = 0.30;
		self.Go.left = not self.Go.left;
		self.Done.left = true; 
		self.isTurnedOn = false;
	end;	
	if InputBinding.hasEvent(InputBinding.LOWER_IMPLEMENT) and self:getIsActiveForInput() and not self.Go.left then	
		self.Speed.back = 3.0;
		self.Go.back = not self.Go.back;
		self.Done.back = true;
		self.jointLimiter = self.Go.back;
	end;	
	if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) and self:getIsActiveForInput() and not self.Go.left then	
		self.isTurnedOn = not self.isTurnedOn;
	end;	

	if self:getIsActive() then
		if self.fixAttacher then
			local implement = self.attacherVehicle:getImplementByObject(self);
			if implement ~= nil then
				local joint = self.attacherVehicle.attacherJoints[implement.jointDescIndex];
				self.saveJoint = self.attacherVehicle.attacherJoints[implement.jointDescIndex];
				self.maxRotJoint = joint.maxRotLimit;
				self.attacherJointIndex = implement.jointDescIndex;
				self.rotationCopy  = joint.minRot;
				joint.minRot = {joint.minRot[1]/2 - Utils.degToRad(7),0,0};			
			--	joint.maxRotLimit = {joint.maxRotLimit[1]/2, 0, 0};
				setJointRotationLimit(joint.jointIndex, 10, true, 0, 0);
			end;
			self.fixAttacher = false;
		end;
		self:setJointRotLimit(self.componentJoints[3],9, 0, 1000, self.jointLimiter, 2,dt);
		self:setJointRotLimit(self.componentJoints[4],9, 0, 1000, self.jointLimiter, 2,dt);
		self:setJointRotLimit(self.componentJoints[5],9, 0, 1000, self.jointLimiter, 2,dt);
		self:setJointRotLimit(self.componentJoints[6],9, 0, 1000, self.jointLimiter, 2,dt);
		self:setJointRotLimit(self.componentJoints[7],9, 0, 1000, self.jointLimiter, 2,dt);
		self:setJointRotLimit(self.componentJoints[8],9, 0, 1000, self.jointLimiter, 2,dt);
		self:setJointRotLimit(self.componentJoints[9],9, 0, 1000, self.jointLimiter, 2,dt);
		self:setJointRotLimit(self.componentJoints[10],9, 0, 1000, self.jointLimiter, 2,dt);
		local joint = self.componentJoints[1];
		setJointFrame(joint.jointIndex, 0,joint.jointNode);		
		local joint = self.componentJoints[2];
		setJointFrame(joint.jointIndex, 0,joint.jointNode);		
		local joint = self.componentJoints[3];
		setJointFrame(joint.jointIndex, 0,joint.jointNode);		
		local joint = self.componentJoints[4];
		setJointFrame(joint.jointIndex, 0,joint.jointNode);				
		local joint = self.componentJoints[5];
		setJointFrame(joint.jointIndex, 0,joint.jointNode);		
		local joint = self.componentJoints[6];
		setJointFrame(joint.jointIndex, 0,joint.jointNode);		
		local joint = self.componentJoints[7];
		setJointFrame(joint.jointIndex, 0,joint.jointNode);		
		local joint = self.componentJoints[8];
		setJointFrame(joint.jointIndex, 0,joint.jointNode);		
		local joint = self.componentJoints[9];
		setJointFrame(joint.jointIndex, 0,joint.jointNode);		
		local joint = self.componentJoints[10];
		setJointFrame(joint.jointIndex, 0,joint.jointNode);
		if self.Go.back then
			--setCenterOfMass(self.components[3].node, 0 ,2.36256, 0);
		else
			--setCenterOfMass(self.components[3].node, 0 ,0, 0);
		end;
		if self.Go.right ~= nil and self.Done.right ~= false then
			self:anim("right", false); 
		end;		
		if self.Go.left ~= nil and self.Done.left ~= false then
			self:anim("left", false); 
		end;		
		if self.Go.back ~= nil and self.Done.back ~= false then
			self:anim("back", false); 
		end;

		
        local hasGroundContact = false;
        local x,y,z = getWorldTranslation(self.groundReferenceNode);
        local terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x, 0, z);
        if terrainHeight+self.groundReferenceThreshold >= y then
            hasGroundContact = true;
        end;
        for k,v in pairs(self.grassParticleSystems) do
            if self.time > v.disableTime then
                Utils.setEmittingState(v.ps, false);
            end;
        end;
        if hasGroundContact then
            local foldAnimTime = self.foldAnimTime;
            if self.isTurnedOn then

                local toFast = self:doCheckSpeedLimit() and self.lastSpeed*3600 > 31;

                if not toFast then
                    local numDropAreas = table.getn(self.tedderDropAreas);
                    for i=1, table.getn(self.cuttingAreas) do
                        local cuttingArea = self.cuttingAreas[i];
                        if foldAnimTime == nil or (foldAnimTime <= cuttingArea.foldMaxLimit and foldAnimTime >= cuttingArea.foldMinLimit) then
                            local x,y,z = getWorldTranslation(cuttingArea.start);
                            local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
                            local x2,y2,z2 = getWorldTranslation(cuttingArea.height);

                            local ratio = g_currentMission.windrowCutLongRatio;

                            local area = Utils.updateFruitCutLongArea(FruitUtil.FRUITTYPE_GRASS, x, z, x1, z1, x2, z2, 0);
                            area = area + Utils.updateFruitCutLongArea(FruitUtil.FRUITTYPE_DRYGRASS, x, z, x1, z1, x2, z2, 0);
                            area = area + Utils.updateFruitWindrowArea(FruitUtil.FRUITTYPE_GRASS, x, z, x1, z1, x2, z2, 0)*ratio;
                            area = area + Utils.updateFruitWindrowArea(FruitUtil.FRUITTYPE_DRYGRASS, x, z, x1, z1, x2, z2, 0)*ratio;
                            if area > 0 then
                                if i <= numDropAreas then
                                    local dropArea = self.tedderDropAreas[i];
                                    local x,y,z = getWorldTranslation(dropArea.start);
                                    local x1,y1,z1 = getWorldTranslation(dropArea.width);
                                    local x2,y2,z2 = getWorldTranslation(dropArea.height);
                                    local old, total = Utils.getFruitCutLongArea(FruitUtil.FRUITTYPE_DRYGRASS, x, z, x1, z1, x2, z2);
                                    area = area + old;
                                    local value = area / total;
                                    if value < 1 and value > 0.1 then
                                        value = 1;
                                    else
                                        value = math.floor(value + 0.6); -- round, biased to the bigger value
                                    end;
                                    if value >= 1 then
                                        value = math.min(value, g_currentMission.maxCutLongValue);
                                        Utils.updateFruitCutLongArea(FruitUtil.FRUITTYPE_DRYGRASS, x, z, x1, z1, x2, z2, value, true);
                                    end;
                                end;
                                local ps = nil;
                                if cuttingArea.grassParticleSystemIndex ~= nil then
                                    ps = self.grassParticleSystems[cuttingArea.grassParticleSystemIndex+1];
                                    if ps ~= nil then
                                        ps.disableTime = self.time + 300;
                                        Utils.setEmittingState(ps.ps, true);
                                    end;
                                end;
                            end;
                        end;
                    end;
                end;
                self.wasToFast = toFast;
            end;
        end;

        if self.isTurnedOn then
            for i=1, table.getn(self.rotors) do
                local rotor = self.rotors[i].index;
                local rotorRot = (-0.008*self.rotors[i].direction)*dt;
                rotate(rotor, 0, rotorRot, 0);
            end;
			if self:getIsActiveForSound() then
				if not self.tedderSoundEnabled and self:getIsActiveForSound() then
					playSample(self.tedderSound, 0, self.tedderSoundVolume, 0);
					setSamplePitch(self.tedderSound, self.tedderSoundPitchOffset);
					self.tedderSoundEnabled = true;
				end;
			end;
        end;

    end;

    if not self.isTurnedOn and self.tedderSoundEnabled then
        stopSample(self.tedderSound);
        self.tedderSoundEnabled = false;
    end;
  
	
end;



function Volto1320t:draw()
	if self.hudActive then
		g_currentMission:addHelpButtonText(g_i18n:getText("Volto1320tOverlay"), InputBinding.Volto1320tOverlay);
	else
		g_currentMission:addHelpButtonText(g_i18n:getText("Volto1320tOverlay"), InputBinding.Volto1320tOverlay);
	end; 
	--if self.hudActive == true then
		--self.hudPDAOverlay:render();
		--setTextBold(true);
		--setTextColor(1, 1, 1, 1.0);
		--renderText(self.hudBasePoxX[1], self.hudBasePoxY[1], 0.020, string.format("%s",tostring(InputBinding.Volto_trsp));
		--renderText(self.hudBasePoxX[2], self.hudBasePoxY[2], 0.020, string.format("%s",tostring(InputBinding.LOWER_IMPLEMENT));
		--renderText(self.hudBasePoxX[3], self.hudBasePoxY[3], 0.020, string.format("%s",tostring(InputBinding.IMPLEMENT_EXTRA));
	--end;

	if self.wasToFast then
		--g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "2", InputBinding.getButtonKeyName(InputBinding.SPEED_LEVEL2)), 0.07+0.022, 0.019+0.029);
		self.hudPDAOverlayL:render();
	end;
end;



function Volto1320t:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)	
	if not resetVehicles then
		
	end; 
    return BaseMission.VEHICLE_LOAD_OK;
end;

function Volto1320t:getSaveAttributesAndNodes(nodeIdent)
	--local attributes = 'trsp="'..tostring(self.trsp)..'" destroy="'..tostring(self.destroy.time)..'"';
	--local attributes = 'trsp="'..tostring(self.trsp)..'" steer="'..tonumber(self.changeWheel)..'" destroy="'..tostring(self.destroy.time)..'"';
	--return attributes, nil;
end;


function Volto1320t:onAttach()
	self.fixAttacher = true; 

end; 	

function Volto1320t:onDetach()
	local joint = self.saveJoint; 
	joint.minRot = self.rotationCopy;
	joint.maxRotLimit = self.maxRotJoint;
    if self.deactivateOnDetach then
        Volto1320t.onDeactivate(self);
    end;
end;
function Volto1320t:onLeave()
    if self.deactivateOnLeave then
        Volto1320t.onDeactivate(self);
    end;
end;
function Volto1320t:onDeactivate()
    for k,v in pairs(self.grassParticleSystems) do
        Utils.setEmittingState(v.ps, false);
    end;
    self.isTurnedOn = false;
    self:onDeactivateSounds();
end;
function Volto1320t:delete()

end;
function Volto1320t:onDeactivateSounds()
    if self.tedderSoundEnabled then
        stopSample(self.tedderSound);
        self.tedderSoundEnabled = false;
    end;
end;

function Volto1320t:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Volto1320t:keyEvent(unicode, sym, modifier, isDown)
end;




